
import com.hazelcast.client.HazelcastClient;
import com.hazelcast.config.Config;
import com.hazelcast.config.MapConfig;
import com.hazelcast.config.MapStoreConfig;
import com.hazelcast.core.Hazelcast;
import com.hazelcast.core.HazelcastInstance;
import com.hazelcast.map.IMap;
import java.time.Duration;
import java.time.Instant;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;


public final class Client {

    private static int DB_WRITE_DELAY_S;
    
    private static final Integer DB_WRITE_DELAY_S_DEFAULT=10;
    private static final String DB_WRITE_DELAY_KEY="DB_WRITE_DELAY";
    private static final String PARMS_INITED_KEY = "PARMS_INITED";
    private static final int N_RECORDS = 10000;
    
    private Client() {
    }
    
    private static void initClusterWideParams(HazelcastInstance hz){
        IMap<String,String> configMap =  hz.getMap("clusterConfigMap");
        String lockKey="1";
        boolean inited = Boolean.parseBoolean(configMap.get(PARMS_INITED_KEY));
        if ( !inited ) {
          configMap.lock(lockKey);
          configMap.put(DB_WRITE_DELAY_KEY, DB_WRITE_DELAY_S_DEFAULT.toString());         
          configMap.put(PARMS_INITED_KEY, Boolean.TRUE.toString());
          configMap.unlock(lockKey);
          
        }        
        DB_WRITE_DELAY_S = Integer.parseInt(configMap.get(DB_WRITE_DELAY_KEY));
    }
    
    private static List<Person> generate(int number) {
        List<Person> res = new ArrayList<>();
        for( int i = 0 ; i < number ; i++){
            String key = ""+ System.nanoTime() + UUID.randomUUID().toString() ;
            String value = "Name number " + String.valueOf(i);
            Person r = new Person(key,value);
            res.add(r);
        }       
        
       return res;
    }

    public static void main(String[] args) throws Exception {
    Config config = new Config();


    HazelcastInstance hz = Hazelcast.newHazelcastInstance(config);
    
    initClusterWideParams(hz);
    
    //configure map to be stored in DB
    System.out.println("Adding Map store to remore MySQL with wrire delay, seconds: "+DB_WRITE_DELAY_S);

    MapConfig mapConfig = new MapConfig("dbMap");
   
    MapStoreConfig mapStoreConfig = new MapStoreConfig();
    mapStoreConfig.setClassName(PersonMapStore.class.getName());
    //set write delay a couple sec to avoid DB performance influence
    mapStoreConfig.setWriteDelaySeconds(DB_WRITE_DELAY_S);
    mapStoreConfig.setOffload(true);
    mapConfig.setMapStoreConfig(mapStoreConfig);
    hz.getConfig().addMapConfig(mapConfig);    
    
    System.out.println("Hazelcast Member instance is running!");


    IMap<String, Person> map = hz.getMap("dbMap");
    
    System.out.println("============ Current map size: "+map.size()+" records ============");
    
    System.out.println("Generating "+N_RECORDS+" records");
      
    List<Person> records = generate(N_RECORDS);

    //put everything into cluster and register execution time
    System.out.println("Putting "+N_RECORDS+" records in the distributed map");

    Instant start = Instant.now();
    
    for( Person p: records){
        map.put(p.id, p);
    }
    
    Instant stop = Instant.now();        
    Duration dur = Duration.between(start,stop);
    
    System.out.println("============ Exiting Client. "+N_RECORDS+" records where added in "+dur.getSeconds()+"."+ dur.getNano()+" seconds. =============" );

    System.out.println("Exiting with proper shutdown. It may take a while...");
    
    Hazelcast.shutdownAll();
    
    System.out.println("Exited safely");
    
    }
}
